// Freeware with no strings attached, witout warranty. Use at you own risk for whatever you want.
// 2018 pcbjunkie.net

// Controller Pin Function
// Looking at the male socket on console (front view, not solder side)
//
// .------------------------------------------.
//  \    [UP] [DOWN] [LEFT] [RIGHT] [+5V]    /
//   \   [B/A] [SELECT] [GND] [C/START]     /
//    '------------------------------------'

// Arduino Pin Definitions
// A6-A7 are not usable 

const int PIN_SELECT = 13;

const int PIN_IN_UP = A0;
const int PIN_IN_DOWN = A1;
const int PIN_IN_LEFT = A2;
const int PIN_IN_RIGHT = A3;
const int PIN_IN_BTN_B_A = A4;
const int PIN_IN_BTN_C_START = A5;

const int PIN_OUT_UP = 2;
const int PIN_OUT_DOWN = 3;
const int PIN_OUT_LEFT = 4;
const int PIN_OUT_RIGHT = 5;
const int PIN_OUT_BTN_1 = 6;
const int PIN_OUT_BTN_2 = 7;
const int PIN_OUT_BTN_3 = 8;
const int PIN_OUT_BTN_4 = 9;
const int PIN_OUT_BTN_5 = 10;
const int PIN_OUT_BTN_6 = 11;
const int PIN_OUT_BTN_START = 12;

// controller select line state
int select;

// button hold counter
int starthold;
const int START_HOLD_TICKS = 100;

void setup() {
  pinMode(PIN_SELECT, OUTPUT);
  
  pinMode(PIN_IN_UP, INPUT_PULLUP);
  pinMode(PIN_IN_DOWN, INPUT_PULLUP);
  pinMode(PIN_IN_LEFT, INPUT_PULLUP);
  pinMode(PIN_IN_RIGHT, INPUT_PULLUP);
  pinMode(PIN_IN_BTN_C_START, INPUT_PULLUP);
  pinMode(PIN_IN_BTN_B_A, INPUT_PULLUP);

  pinMode(PIN_OUT_UP, OUTPUT);
  pinMode(PIN_OUT_DOWN, OUTPUT);
  pinMode(PIN_OUT_LEFT, OUTPUT);
  pinMode(PIN_OUT_RIGHT, OUTPUT);
  pinMode(PIN_OUT_BTN_1, OUTPUT);
  pinMode(PIN_OUT_BTN_2, OUTPUT);
  pinMode(PIN_OUT_BTN_3, OUTPUT);
  pinMode(PIN_OUT_BTN_4, OUTPUT);
  pinMode(PIN_OUT_BTN_5, OUTPUT);
  pinMode(PIN_OUT_BTN_6, OUTPUT);
  pinMode(PIN_OUT_BTN_START, OUTPUT);

  select = 0;
  Serial.begin(230400); 
}

void loop() {
  // start with select pin high and let things settle
  select = 1;
  digitalWrite(PIN_SELECT, select);
  delay(10);

  /*
  Pin  Select: low  Select: high  Select: pulse-3
  --- ------------  ------------  ---------------
    1  joypad up     joypad up     button Z
    2  joypad down   joypad down   button Y
    3  logic low     joypad left   button X
    4  logic low     joypad right
    6  button A      button B
    9  start button  button C
  */

  // read the UP and DOWN values during the select high portion
  digitalWrite(PIN_OUT_UP, digitalRead(PIN_IN_UP));
  digitalWrite(PIN_OUT_DOWN, digitalRead(PIN_IN_DOWN));

  // SELECT is still high
  digitalWrite(PIN_OUT_BTN_2, digitalRead(PIN_IN_BTN_B_A));
  digitalWrite(PIN_OUT_BTN_3, digitalRead(PIN_IN_BTN_C_START));
  digitalWrite(PIN_OUT_LEFT, digitalRead(PIN_IN_LEFT));
  digitalWrite(PIN_OUT_RIGHT, digitalRead(PIN_IN_RIGHT));

  print_state();

  // set select pin low and let things settle
  select = 0;
  digitalWrite(PIN_SELECT, select);
  delay(10);

  // now SELECT is low
  digitalWrite(PIN_OUT_BTN_1, digitalRead(PIN_IN_BTN_B_A));
  digitalWrite(PIN_OUT_BTN_START, digitalRead(PIN_IN_BTN_C_START));    

  print_state();

  // now trigger the 3-pulse mode of the controller
  select = 3;
  digitalWrite(PIN_SELECT, 1);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 0);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 1);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 0);
  delayMicroseconds(15);
  digitalWrite(PIN_SELECT, 1);
  delayMicroseconds(15);
  //digitalWrite(PIN_SELECT, 0);
  //delayMicroseconds(15);

  digitalWrite(PIN_OUT_BTN_4, digitalRead(PIN_IN_LEFT));
  digitalWrite(PIN_OUT_BTN_5, digitalRead(PIN_IN_DOWN));
  digitalWrite(PIN_OUT_BTN_6, digitalRead(PIN_IN_UP));

  // print state via serial (can be disabled)  
  print_state();
}

void print_state(){
  //Serial.print("*-UDLRBC-UDLR123456S\n"); 

  if (select == 0) Serial.print("L");
  else if (select == 1) Serial.print("H"); 
  else Serial.print("3");

  Serial.print("-");

  digitalRead(PIN_IN_UP) ? Serial.print("U") : Serial.print(" ");
  digitalRead(PIN_IN_DOWN) ? Serial.print("D") : Serial.print(" ");
  digitalRead(PIN_IN_LEFT) ? Serial.print("L") : Serial.print(" ");
  digitalRead(PIN_IN_RIGHT) ? Serial.print("R") : Serial.print(" ");

  digitalRead(PIN_IN_BTN_B_A) ? Serial.print("B") : Serial.print(" ");
  digitalRead(PIN_IN_BTN_C_START) ? Serial.print("C") : Serial.print(" ");
  
  Serial.print("-");

  digitalRead(PIN_OUT_UP) ? Serial.print("U") : Serial.print(" ");
  digitalRead(PIN_OUT_DOWN) ? Serial.print("D") : Serial.print(" ");
  digitalRead(PIN_OUT_LEFT) ? Serial.print("L") : Serial.print(" ");
  digitalRead(PIN_OUT_RIGHT) ? Serial.print("R") : Serial.print(" ");

  digitalRead(PIN_OUT_BTN_1) ? Serial.print("1") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_2) ? Serial.print("2") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_3) ? Serial.print("3") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_4) ? Serial.print("4") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_5) ? Serial.print("5") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_6) ? Serial.print("6") : Serial.print(" ");
  digitalRead(PIN_OUT_BTN_START) ? Serial.print("S") : Serial.print(" ");

  Serial.print("\n"); 
}

